﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using MapApp.Model;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.Net;


namespace MapApp
{
    public class HistoryFileHttp : HistoryFile
    {
        private static string BASE_URL = "http://m.rkdrm.com/uploads/";
        private static string PUT_URL = "http://m.rkdrm.com/u.php";


        public HistoryFileHttp()
        {
            HistoryFileLoader = LoadHistoryFile;
            HistoryFileSaver = SaveHistoryFile;
        }

        private List<string> LoadHistoryFile(string fileName)
        {
            WebClient client = new WebClient();
            Stream stream = client.OpenRead(BASE_URL + fileName);
            BinaryFormatter bf = new BinaryFormatter();
            List<string> result = (List<string>)bf.Deserialize(stream);
            stream.Close();
            return result;
        }

        private void SaveHistoryFile(List<string> contents, string fileName)
        {
            // Don't try to save empty history
            if (contents == null) return;
            if (contents.Count == 0) return;

            MemoryStream memoryStream = new MemoryStream();
            BinaryFormatter bf = new BinaryFormatter();
            bf.Serialize(memoryStream, contents);

            byte[] bytes = memoryStream.ToArray();
            memoryStream.Close();

            string sResponse = UploadDataEx(bytes, fileName,
                PUT_URL, "uploadedfile", "application/octet-stream",
                null, null);

        }


        public static string UploadDataEx(byte[] data, string uploadfileName, string url,
    string fileFormName, string contenttype, NameValueCollection querystring,
    CookieContainer cookies)
        {
            if ((fileFormName == null) ||
                (fileFormName.Length == 0))
            {
                fileFormName = "file";
            }

            if ((contenttype == null) ||
                (contenttype.Length == 0))
            {
                contenttype = "application/octet-stream";
            }


            string postdata;
            postdata = "?";
            if (querystring != null)
            {
                foreach (string key in querystring.Keys)
                {
                    postdata += key + "=" + querystring.Get(key) + "&";
                }
            }
            Uri uri = new Uri(url + postdata);


            string boundary = "----------" + DateTime.Now.Ticks.ToString("x");
            HttpWebRequest webrequest = (HttpWebRequest)WebRequest.Create(uri);
            webrequest.CookieContainer = cookies;
            webrequest.ContentType = "multipart/form-data; boundary=" + boundary;
            webrequest.Method = "POST";


            // Build up the post message header
            StringBuilder sb = new StringBuilder();
            sb.Append("--");
            sb.Append(boundary);
            sb.Append("\r\n");
            sb.Append("Content-Disposition: form-data; name=\"");
            sb.Append(fileFormName);
            sb.Append("\"; filename=\"");
            sb.Append(uploadfileName);
            //sb.Append(Path.GetFileName(uploadfileName));
            sb.Append("\"");
            sb.Append("\r\n");
            sb.Append("Content-Type: ");
            sb.Append(contenttype);
            sb.Append("\r\n");
            sb.Append("\r\n");

            string postHeader = sb.ToString();
            byte[] postHeaderBytes = Encoding.UTF8.GetBytes(postHeader);

            // Build the trailing boundary string as a byte array
            // ensuring the boundary appears on a line by itself
            byte[] boundaryBytes = Encoding.ASCII.GetBytes("\r\n--" + boundary + "\r\n");

            MemoryStream memoryStream = new MemoryStream(data);

            //FileStream fileStream = new FileStream(uploadfileName, FileMode.Open, FileAccess.Read);
            //long length = postHeaderBytes.Length + fileStream.Length + boundaryBytes.Length;
            long length = postHeaderBytes.Length + memoryStream.Length + boundaryBytes.Length;
            webrequest.ContentLength = length;

            Stream requestStream = webrequest.GetRequestStream();

            // Write out our post header
            requestStream.Write(postHeaderBytes, 0, postHeaderBytes.Length);

            // Write out the file contents
            ////byte[] buffer = new Byte[checked((uint)Math.Min(4096, (int)fileStream.Length))];
            byte[] buffer = new Byte[checked((uint)Math.Min(4096, (int)memoryStream.Length))];
            int bytesRead = 0;
            //////////while ((bytesRead = fileStream.Read(buffer, 0, buffer.Length)) != 0)
            while ((bytesRead = memoryStream.Read(buffer, 0, buffer.Length)) != 0)
                requestStream.Write(buffer, 0, bytesRead);

            // Write out the trailing boundary
            requestStream.Write(boundaryBytes, 0, boundaryBytes.Length);
            WebResponse response = webrequest.GetResponse();
            Stream s = response.GetResponseStream();
            StreamReader sr = new StreamReader(s);

            return sr.ReadToEnd();
        }

    }
}
