﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Text;

using System.Runtime.InteropServices;
using MsHtmHstInterop;

using MapApp.Model;

namespace MapApp.Ui.Controls
{
    public partial class GoogleStreetViewControl : System.Windows.Forms.WebBrowser
    {
        private static string _streetViewFilePath = AppUtil.tempDir + "/streetview.html";
        private MapLocation _initialLocation;


        public GoogleStreetViewControl()
        {
            InitializeComponent();
            CustomInitialize();
        }

        public GoogleStreetViewControl(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
            CustomInitialize();
        }

        private void CustomInitialize()
        {
            AppUtil.writeResourceToDisk("MapApp.Ui.Controls.Resources.html.streetview.html",
                _streetViewFilePath);

            DocumentCompleted += new System.Windows.Forms.WebBrowserDocumentCompletedEventHandler(GoogleStreetViewControl_DocumentCompleted);
            Url = new Uri(_streetViewFilePath);
        }

        void GoogleStreetViewControl_DocumentCompleted(object sender, System.Windows.Forms.WebBrowserDocumentCompletedEventArgs e)
        {
            // Register the callback handler (for window.external calls in javascript):
            ICustomDoc cDoc = (ICustomDoc)((SHDocVw.WebBrowser)this.ActiveXInstance).Document;
            cDoc.SetUIHandler(new StreetViewDocHostUIHandler(this));

            if (_initialLocation != null)
            {
                SetLocation(_initialLocation);
            }
        }

        #region Properties

        public MapLocation InitialLocation
        {
            get { return _initialLocation; }
        }

        #endregion Properties

        #region Public Methods
        /// <summary>
        /// Sould never be called in code - only used by form constructor with 
        /// and initial location argument
        /// </summary>
        public void SetInitialLocation()
        {
            if (InitialLocation != null)
            {
                Object[] objArray = { InitialLocation.Lat, InitialLocation.Lng };
                Document.InvokeScript("setInitialLocation", objArray);
            }
        }

        public void SetLocation(MapLocation loc)
        {
            if (_initialLocation == null)
            {
                _initialLocation = loc;
            }
            Object[] objArray = { loc.Lat, loc.Lng};
            Document.InvokeScript("setLocation", objArray);
        }

        #endregion // Public Methods

    }





    public class StreetViewExternal : IStreetViewExternalInterface
    {
        //private Map _map;
        private GoogleStreetViewControl _streetViewControl;
        public StreetViewExternal(GoogleStreetViewControl streetViewControl)
        {
            _streetViewControl = streetViewControl;
        }

        #region IStreetViewExternalInterface Members

        public void getInitialLocation()
        {
            _streetViewControl.SetInitialLocation();
        }

        //////public void pushZoomHistory(double lat, double lng, int zoomLevel)
        //////{
        //////    _mapControl.ZoomHistory.Push(new MapLocation("", lat, lng, zoomLevel));
        //////    //AppUi.StatusText = "Zoom: " + lat + ", " + lng + ", " + zoomLevel;
        //////}

        //////public void addAllMarkers()
        //////{
        //////    if (_mapControl.Map.MapLocations != null)
        //////    {
        //////        foreach (MapLocation loc in _mapControl.Map.MapLocations)
        //////        {
        //////            loc.Id = _mapControl.NewMarker(loc);
        //////        }
        //////    }
        //////}

        //////public void markerMoved(int markerId, double toLat, double toLng)
        //////{
        //////    if (_mapControl.Map.MapLocations != null)
        //////    {
        //////        // TODO: Need to implement a dictionary of locationId to location for lookup
        //////        MapLocation loc = _mapControl.Map.MapLocations.Find(markerId);
        //////        if (loc != null)
        //////        {
        //////            loc.Lat = toLat;
        //////            loc.Lng = toLng;
        //////        }
        //////    }
        //////}

        //////public void nonMarkerContextMenu(int pixelX, int pixelY)
        //////{
        //////    Point mousePosition = new Point(pixelX, pixelY);
        //////    _mapControl.RightClickMousePosition = mousePosition;
        //////    //_rightClickMapLocation = e.MapLocation;
        //////    int currentZoomLevel = _mapControl.GetCurrentZoomLevel();
        //////    MapLocation rightClickLocation = _mapControl.PixelToMapLocation(pixelX, pixelY);
        //////    _mapControl.RightClickMapLocation = new MapLocation("", rightClickLocation.Lat,
        //////            rightClickLocation.Lng, currentZoomLevel);
        //////    if (_mapControl.Map.MapLocations.Count > 0)
        //////    {
        //////        _mapControl.zoomToAllMarkersContextMenuItem.Visible = true;
        //////        _mapControl.showHideLabelsContextMenuItem.Visible = true;
        //////    }
        //////    else
        //////    {
        //////        _mapControl.zoomToAllMarkersContextMenuItem.Visible = false;
        //////        _mapControl.showHideLabelsContextMenuItem.Visible = false;

        //////    }

        //////    _mapControl.nonMarkerContextMenuStrip.Show(_mapControl, mousePosition);

        //////    //throw new NotImplementedException();
        //////}


        //////public void markerContextMenu(int markerId, int pixelX, int pixelY)
        //////{
        //////    _mapControl.RightClickMousePosition = new Point(pixelX, pixelY);
        //////    //_rightClickMapLocation = e.MapLocation;
        //////    int currentZoomLevel = _mapControl.GetCurrentZoomLevel();
        //////    MapLocation rightClickLocation = _mapControl.PixelToMapLocation(pixelX, pixelY);
        //////    _mapControl.RightClickMapLocation = new MapLocation("", rightClickLocation.Lat,
        //////            rightClickLocation.Lng, currentZoomLevel);
        //////    _mapControl.SelectedMarker = _mapControl.Map.MapLocations.Find(markerId);
        //////    _mapControl.markerContextMenuStrip.Show(_mapControl, new Point(pixelX, pixelY));

        //////    //throw new NotImplementedException();
        //////}

        #endregion
    }

    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface IStreetViewExternalInterface
    {
        [DispId(0)]
        void getInitialLocation();
    }

    public class StreetViewDocHostUIHandler : IDocHostUIHandler
    {
        private GoogleStreetViewControl _streetViewControl;
        public StreetViewDocHostUIHandler(GoogleStreetViewControl streetViewControl)
        {
            _streetViewControl = streetViewControl;
        }

        // IDocHostUIHandler implementation

        void IDocHostUIHandler.EnableModeless(int fEnable)
        {

        }

        void IDocHostUIHandler.FilterDataObject(MsHtmHstInterop.IDataObject pDO, out MsHtmHstInterop.IDataObject ppDORet)
        {
            ppDORet = null;
        }

        void IDocHostUIHandler.GetDropTarget(MsHtmHstInterop.IDropTarget pDropTarget, out MsHtmHstInterop.IDropTarget ppDropTarget)
        {
            ppDropTarget = null;
        }

        void IDocHostUIHandler.GetExternal(out object ppDispatch)
        {
            ppDispatch = new StreetViewExternal(_streetViewControl);
        }

        void IDocHostUIHandler.GetHostInfo(ref _DOCHOSTUIINFO pInfo)
        {

        }

        void IDocHostUIHandler.GetOptionKeyPath(out string pchKey, uint dw)
        {
            pchKey = null;
        }

        void IDocHostUIHandler.HideUI()
        {

        }

        void IDocHostUIHandler.OnDocWindowActivate(int fActivate)
        {

        }

        void IDocHostUIHandler.OnFrameWindowActivate(int fActivate)
        {

        }

        void IDocHostUIHandler.ResizeBorder(ref MsHtmHstInterop.tagRECT prcBorder, IOleInPlaceUIWindow pUIWindow, int fRameWindow)
        {

        }

        void IDocHostUIHandler.ShowContextMenu(uint dwID, ref MsHtmHstInterop.tagPOINT ppt, object pcmdtReserved, object pdispReserved)
        {

        }

        void IDocHostUIHandler.ShowUI(uint dwID, IOleInPlaceActiveObject pActiveObject, IOleCommandTarget pCommandTarget, IOleInPlaceFrame pFrame, IOleInPlaceUIWindow pDoc)
        {

        }

        void IDocHostUIHandler.TranslateAccelerator(ref tagMSG lpmsg, ref Guid pguidCmdGroup, uint nCmdID)
        {

        }

        void IDocHostUIHandler.TranslateUrl(uint dwTranslate, ref ushort pchURLIn, IntPtr ppchURLOut)
        {

        }

        void IDocHostUIHandler.UpdateUI()
        {

        }
    }





}
