using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using System.Net;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;

using MapApp.Model;

namespace MapApp.Ui.Controls
{
    public partial class FrmFind : AppFormBase
    {
        //private HistoryFile _historyFile = new HistoryFileHttp();
        private HistoryFile _historyFile = new HistoryFileIsolatedStorage();
        private static string _mapOwnerName = "rk";
        static string addressHistoryFileName = "FindHistory.dat";
        //static string historyFilePath = AppUtil.tempDir + "/" + addressHistoryFileName;
        //static string historyFilePath = "http://interchart.com/" + addressHistoryFileName;

        Thread historyThread; //= new Thread(new ThreadStart(HistoryWorker));

        public FrmFind()
        {
            InitializeComponent();
            string findFilePath = AppUtil.tempDir + "/find.html";
            AppUtil.writeResourceToDisk("MapApp.Ui.Controls.Resources.html.find.html", findFilePath);
            wbFind.Url = new Uri(findFilePath);

            cmbFind.AutoCompleteCustomSource = new AutoCompleteStringCollection();

            historyThread = new Thread(new ThreadStart(HistoryWorker));
            historyThread.Start();

            //LoadHistoryFile();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void wbFind_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            wbFind.Document.GetElementById("commsDiv").Click += new HtmlElementEventHandler(link_Click);
        }

        private void link_Click(object sender, HtmlElementEventArgs e)
        {
            string[] parts = wbFind.Document.GetElementById("commsDiv").InnerHtml.Split(':');
            string addr = wbFind.Document.GetElementById("addrDiv").InnerText;
            MapLocation zoomToLoc = new MapLocation(addr, Double.Parse(parts[0]), Double.Parse(parts[1]),
                    Int32.Parse(parts[2]));
            zoomToLoc.Label = addr;
            if (!cmbFind.AutoCompleteCustomSource.Contains(zoomToLoc.LocationString))
            {
                cmbFind.AutoCompleteCustomSource.Add(zoomToLoc.LocationString);
            }

            AppUi.MapControl.AddNewMarker(zoomToLoc);
            AppUi.Map.ZoomTo(zoomToLoc);
        }

        private void cmbFind_KeyUp(object sender, KeyEventArgs e)
        {
            // If the ENTER key is pressed, the Handled property is set to true, 
            // to indicate the event is handled.
            if (e.KeyCode == Keys.Enter)
            {
                doSearch();
                // Tell system that the event was handled
                e.Handled = true;
            }

        }

        private void doSearch()
        {
            try
            {
                // Do the search:
                string searchText = cmbFind.Text;
                if (chkAustralia.Checked) searchText += " Australia";
                if (chkUSA.Checked) searchText += " USA";
                wbFind.Document.GetElementById("search").InnerText = searchText;
                Object obj = wbFind.Document.GetElementById("enter").DomElement;
                Type type = obj.GetType();
                MethodInfo[] methods = obj.GetType().GetMethods();
                MethodInfo mi = obj.GetType().GetMethod("click");
                mi.Invoke(obj, new object[0]);
            }
            catch { } // ignore exceptions
        }

        private void chk_CheckedChanged(object sender, EventArgs e)
        {
            if ((sender == chkAustralia) && chkAustralia.Checked)
            {
                chkUSA.Checked = false;
            }
            if ((sender == chkUSA) && chkUSA.Checked)
            {
                chkAustralia.Checked = false;
            }
        }

        private void btnFind_Click(object sender, EventArgs e)
        {
            doSearch();
        }


        ////////////////////////////////////////////////////// History functions ///////////////////

        // Executes in historyThread
        void HistoryWorker()
        {
            LoadHistoryFile();
        }


        private void LoadHistoryFile()
        {
            if (cmbFind.AutoCompleteCustomSource != null)
            {
                try
                {
                    List<string> historyList = _historyFile.Load(addressHistoryFileName);
                    foreach (string addr in historyList)
                    {
                        cmbFind.AutoCompleteCustomSource.Add(addr);
                        cmbFind.Items.Add(addr);
                        //object[] args = {addr};
                        //cmbFind.BeginInvoke(new AddLineToDropdownDelegate(AddLineToDropdownDelegateMethod), args);
                    }
                }
                catch (Exception ex)
                {
                    AppUi.StatusText = ex.Message;
                }
            }
        }

        //public delegate void AddLineToDropdownDelegate(string line);
        //public void AddLineToDropdownDelegateMethod(string line)
        //{
        //    if ((cmbFind != null) && (cmbFind.AutoCompleteCustomSource != null)) {
        //    cmbFind.AutoCompleteCustomSource.Add(line);
        //    }
        //}



        /// <summary>
        /// Serialize the location history to persistent storage when the form closes
        /// Uses Amazon S3.
        /// </summary>
        public void SaveHistoryFile()
        {
            try
            {
                if (cmbFind.AutoCompleteCustomSource != null)
                {
                    //HistoryFile historyFile = new HistoryFileHttp();
                    List<string> historyList = new List<string>();
                    foreach (string addr in cmbFind.AutoCompleteCustomSource)
                    {
                        historyList.Add(addr);
                    }
                    _historyFile.Save(historyList, addressHistoryFileName);
                }
            }
            catch (Exception ex)
            {
                AppUi.StatusText = ex.Message;
            }

        }

        private void FrmFind_Load(object sender, EventArgs e)
        {
            cmbFind.AutoCompleteSource = AutoCompleteSource.CustomSource;
        }

    }
}